'From Haver 5.0 [latest update: #4530] on 27 February 2021 at 10:32:34 pm'!
'Description I contain subclasses of Symbol and IdentityDictionary that aware the environment a symbol belongs to.'!
!provides: 'EnvironmentAwareness' 1 25!
!requires: 'Cuis-Base' 50 4392 nil!
SystemOrganization addCategory: 'EnvironmentAwareness'!


!classDefinition: #EnvironmentBindingIdentityDictionary category: 'EnvironmentAwareness'!
IdentityDictionary subclass: #EnvironmentBindingIdentityDictionary
	instanceVariableNames: 'environment'
	classVariableNames: 'Symbols2Environment'
	poolDictionaries: ''
	category: 'EnvironmentAwareness'!
!classDefinition: 'EnvironmentBindingIdentityDictionary class' category: 'EnvironmentAwareness'!
EnvironmentBindingIdentityDictionary class
	instanceVariableNames: ''!

!classDefinition: #GlobalBindingIdentityDictionary category: 'EnvironmentAwareness'!
IdentityDictionary subclass: #GlobalBindingIdentityDictionary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EnvironmentAwareness'!
!classDefinition: 'GlobalBindingIdentityDictionary class' category: 'EnvironmentAwareness'!
GlobalBindingIdentityDictionary class
	instanceVariableNames: ''!

!classDefinition: #SymbolManager category: 'EnvironmentAwareness'!
Object subclass: #SymbolManager
	instanceVariableNames: 'symbol2environment environments2symbols'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EnvironmentAwareness'!
!classDefinition: 'SymbolManager class' category: 'EnvironmentAwareness'!
SymbolManager class
	instanceVariableNames: 'uniqueInstance'!


!EnvironmentBindingIdentityDictionary commentStamp: '<historical>' prior: 0!
I am an IdentityDictionary that binds its keys to the environment
specific symbol namespace.

Needless to say that this is only done for keys that are symbols.!

!GlobalBindingIdentityDictionary commentStamp: '<historical>' prior: 0!
I am an IdentityDictionary that binds its keys to the global symbol namespace.

Needless to say that this is only done for keys that are symbols.!

!SymbolManager commentStamp: '<historical>' prior: 0!
I am the manager for environment specific symbols.

symbol2environment: maps (uninterned) symbols to environments.
environments2symbols: maps environments to weak value dictionaries with their symbols.!

!EnvironmentBindingIdentityDictionary methodsFor: 'accessing' stamp: 'KLG 10/9/2020 21:56:13'!
at: environmentUnawareKey put: anObject
	"Set the value at key to be anObject. 
	If key is not found, create a new entry for key and set is value to anObject.
	If key is found, update the existing association.
	Answer anObject."

	| index assoc key |
	index _ super findElementOrNil: 
		(key _ environmentUnawareKey forEnvironmentIfAware: environment).
	assoc _ array at: index.
	assoc
		ifNil: [ self atNewIndex: index put: (Association key: key value: anObject) ]
		ifNotNil: [ assoc value: anObject ].
	^ anObject! !

!EnvironmentBindingIdentityDictionary methodsFor: 'accessing' stamp: 'KLG 10/10/2020 20:20:36'!
environment
	"Answer the value of environment"

	^ environment! !

!EnvironmentBindingIdentityDictionary methodsFor: 'accessing' stamp: 'KLG 10/28/2020 18:24:35'!
environment: anObject
	"Set the value of environment"

	self ifNotEmpty: [ ^ self error: 'I must be empty to set my environment' ].
	environment _ anObject! !

!EnvironmentBindingIdentityDictionary methodsFor: 'private' stamp: 'KLG 10/9/2020 21:48:04'!
scanFor: anObject
	"Scan the key array for the first slot containing either a nil (indicating an empty slot) or an element that matches anObject. Answer the index of that slot or zero if no slot is found. This method will be overridden in various subclasses that have different interpretations for matching elements.

Search for an environment aware version of anObject."
	
	^ super scanFor: (anObject forEnvironmentIfAware: environment)! !

!EnvironmentBindingIdentityDictionary methodsFor: 'adding' stamp: 'KLG 12/13/2020 17:59:47'!
add: anAssociation
	"Quite like doing
		self at: anAssociation key put: anAssociation value
	but making sure the argument is stored.
	This method should be used when the argument is stored elsewhere,
	and its value should be kept in sync with the value stored in me.

	If the key already exists, and it is desired to keep the existing association, then call #at:put:
	
	We may copy the association passed in."

	| index key keyInEnvironment associationToAdd |
	Transcript
		cr;
		show: 'add: ';
		show: anAssociation.
	keyInEnvironment _ (key _ anAssociation key) forEnvironmentIfAware: environment.
	keyInEnvironment == key
		ifTrue: [ 
			Transcript show: ' KEEP'.
			associationToAdd _ anAssociation ]
		ifFalse: [
			Transcript show: ' copied'.
			associationToAdd _ anAssociation shallowCopy.
			associationToAdd key: keyInEnvironment ].
	index _ super findElementOrNil: keyInEnvironment.
	(array at: index)
		ifNil: [
			self
				atNewIndex: index
				put: associationToAdd ]
		ifNotNil: [
			array
				at: index
				put: associationToAdd ].
	^ associationToAdd! !

!EnvironmentBindingIdentityDictionary methodsFor: 'adding' stamp: 'KLG 12/13/2020 18:00:29'!
declare: key from: aDictionary
	"Add key to the receiver. If key already exists, do nothing. If aDictionary 
	includes key, then remove it from aDictionary and use its association as 
	the element of the receiver. Answer it.
	If the key wasn't in the receiver or aDictionary, use nil as the value."

	| association |
	(self includesKey: key) ifTrue: [^ nil].
	(aDictionary includesKey: key)
		ifTrue: [
			association _ aDictionary associationAt: key.
			association _ self add: association.
			aDictionary removeKey: key.
			^ association ]
		ifFalse: [
			self at: key put: nil.
			^ nil ]! !

!EnvironmentBindingIdentityDictionary class methodsFor: 'instance creation' stamp: 'KLG 10/10/2020 20:22:15'!
forEnvironment: anEnvironment
	"Create the dictionary for an environment."

	^ self new environment: anEnvironment; yourself! !

!GlobalBindingIdentityDictionary methodsFor: 'private' stamp: 'KLG 10/11/2020 15:12:40'!
scanFor: anObject
	"Scan the key array for the first slot containing either a nil (indicating an empty slot) or an element that matches anObject. Answer the index of that slot or zero if no slot is found. This method will be overridden in various subclasses that have different interpretations for matching elements.

Search for an environment aware version of anObject."
	
	^ super scanFor: anObject forGlobalNamespace! !

!GlobalBindingIdentityDictionary methodsFor: 'accessing' stamp: 'KLG 10/11/2020 15:13:30'!
at: environmentAwareKey put: anObject
	"Set the value at key to be anObject. 
	If key is not found, create a new entry for key and set is value to anObject.
	If key is found, update the existing association.
	Answer anObject."

	| index assoc key |
	index _ super findElementOrNil: 
		(key _ environmentAwareKey forGlobalNamespace).
	assoc _ array at: index.
	assoc
		ifNil: [ self atNewIndex: index put: (Association key: key value: anObject) ]
		ifNotNil: [ assoc value: anObject ].
	^ anObject! !

!GlobalBindingIdentityDictionary methodsFor: 'adding' stamp: 'KLG 10/11/2020 15:15:08'!
add: anAssociation
	"Quite like doing
		self at: anAssociation key put: anAssociation value
	but making sure the argument is stored.
	This method should be used when the argument is stored elsewhere,
	and its value should be kept in sync with the value stored in me.

	If the key already exists, and it is desired to keep the existing association, then call #at:put:
	
	We' l modify the association passed in"

	| index key |
	anAssociation key: (key _ anAssociation key forGlobalNamespace).
	index _ super findElementOrNil: key.
	(array at: index)
		ifNotNil: [ array at: index put: anAssociation ]
		ifNil: [ self atNewIndex: index put: anAssociation ].
	^ anAssociation! !

!GlobalBindingIdentityDictionary class methodsFor: 'instance creation' stamp: 'KLG 10/11/2020 15:09:18'!
forEnvironment: anEnvironment
	"Create the dictionary for an environment."

	^ self new environment: anEnvironment; yourself! !

!SymbolManager methodsFor: 'initialization' stamp: 'KLG 11/13/2020 19:56:47'!
initialize
	"Initialize the dictionaries."

	super initialize.
	symbol2environment _ WeakIdentityKeyDictionary new.
	environments2symbols _ WeakIdentityKeyDictionary new.! !

!SymbolManager methodsFor: 'instance creation' stamp: 'KLG 12/4/2020 17:38:01'!
ensureSymbol: aSymbol forEnvironment: anEnvironment
	"Answer a symbol for anEnvironment"

	| answer |
	[ answer ] whileNil: [ | string string2symbol |
		answer _ (string2symbol _ environments2symbols
			at: anEnvironment
			ifAbsentPut: [		WeakValueDictionary new ])
				at: (string _ aSymbol asString)
				ifAbsentPut: [ | newSymbol |
					symbol2environment 
						at:  (	newSymbol _ Symbol new: string size :: string: string)
						put: anEnvironment.
						newSymbol ].
		answer ifNil: [ string2symbol removeKey: string ] ].
	^ answer! !

!SymbolManager methodsFor: 'accessing' stamp: 'KLG 11/13/2020 20:26:36'!
environmentFor: aSymbol
	"Answer the environment of aSymbol.
	
	Answer nil if there is none."

	^ symbol2environment at: aSymbol ifAbsent: [ nil ]! !

!SymbolManager methodsFor: 'evaluation' stamp: 'KLG 2/27/2021 22:29:46'!
forEnvironmentOfSymbol: aSymbol do: aBlock
	"If aSymbol exists in anEnvironment evaluate a Block with the environment."

	aBlock value: (symbol2environment at: aSymbol ifAbsent: [ ^ self ] :: ifNil: [ ^ self ])				! !

!SymbolManager methodsFor: 'evaluation' stamp: 'KLG 11/13/2020 20:42:52'!
forSymbol: aSymbol inEnvironment: anEnvironment do: aBlock
	"If aSymbol exists in anEnvironment evaluate a Block with the symbol."

	environments2symbols
		at: anEnvironment
		ifAbsent: [		^ self ] ::
			at: aSymbol asString
			ifPresent: aBlock
				! !

!SymbolManager class methodsFor: 'instance creation' stamp: 'KLG 11/13/2020 20:32:36'!
ensureSymbol: aSymbol forEnvironment: anEnvironment
	"Answer a symbol for anEnvironment"

	^ self uniqueInstance ensureSymbol: aSymbol forEnvironment: anEnvironment! !

!SymbolManager class methodsFor: 'reset' stamp: 'KLG 11/13/2020 20:54:44'!
resetEnvironments
	"Reset all environment specific symbols."

	uniqueInstance _ nil! !

!SymbolManager class methodsFor: 'accessing' stamp: 'KLG 11/13/2020 20:27:02'!
environmentFor: aSymbol
	"Answer the environment of aSymbol.
	
	Answer nil if there is none."

	^ self uniqueInstance environmentFor: aSymbol! !

!SymbolManager class methodsFor: 'accessing' stamp: 'KLG 11/13/2020 19:59:16'!
uniqueInstance
	"Answer my unique instance."

	^ uniqueInstance ifNil: [ uniqueInstance _ self new ]! !

!SymbolManager class methodsFor: 'evaluation' stamp: 'KLG 11/13/2020 22:41:00'!
forEnvironmentOfSymbol: aSymbol do: aBlock
	"If aSymbol exists in anEnvironment evaluate a Block with the environment."

	^ self uniqueInstance forEnvironmentOfSymbol: aSymbol do: aBlock! !

!SymbolManager class methodsFor: 'evaluation' stamp: 'KLG 11/13/2020 20:44:04'!
forSymbol: aSymbol inEnvironment: anEnvironment do: aBlock
	"If aSymbol exists in anEnvironment evaluate a Block with the symbol."

	^ self uniqueInstance forSymbol: aSymbol inEnvironment: anEnvironment do: aBlock! !

!ProtoObject methodsFor: '*EnvironmentAwareness' stamp: 'KLG 10/11/2020 22:05:58'!
doWithEnvironmentIfAware: aBlock
	"Evaluate aBlock with my environment, if I am environment aware.
	
	Do nothing here, because I don't know nothing about no environments."! !

!ProtoObject methodsFor: '*EnvironmentAwareness' stamp: 'KLG 10/9/2020 21:36:59'!
forEnvironmentIfAware: anEnvironment
	"Answer an environment version of mine.

	I am not environment aware, so answer me."! !

!ProtoObject methodsFor: '*EnvironmentAwareness' stamp: 'KLG 10/11/2020 15:11:08'!
forGlobalNamespace
	"Answer global version of mine.

	I am not environment aware, that is I am always global, so answer me."! !

!Symbol methodsFor: '*EnvironmentAwareness' stamp: 'KLG 12/9/2020 12:07:15'!
compare: symbol1 with: symbol2 collated: order
	"Return 1, 2 or 3, if string1 is <, =, or > string2, with the collating order of characters given by the order array."

	| answer |
	answer _ super compare: symbol1 with: symbol2 collated: order.
	answer = 2 and: [ symbol1 isSymbol ] and: [ symbol2  isSymbol ] :: ifTrue: [ | e1 |
		(e1 _ symbol1 environment)
			ifNil: [ symbol2 environment ifNotNil: [ answer _ 1 ] ]
			ifNotNil: [ | e2 | 
				(e2 _ symbol2 environment) 
					ifNil: [ answer _ 3 ]
					ifNotNil: [ 
						answer _ 
							self
								"In the case of our environment implementations #printString includes
								the environment manager's name"
								compare: e1 printString
								with: e2 printString
								collated: order ] ] ].
	^ answer! !

!Symbol methodsFor: '*EnvironmentAwareness' stamp: 'KLG 11/13/2020 22:39:50'!
doWithEnvironmentIfAware: aBlock
	"Evaluate aBlock with my environment, if I am environment aware."
	
	^ SymbolManager forEnvironmentOfSymbol: self do: aBlock
! !

!Symbol methodsFor: '*EnvironmentAwareness' stamp: 'KLG 11/13/2020 20:24:02'!
environment
	"Answer my environment."

	^ SymbolManager environmentFor: self! !

!Symbol methodsFor: '*EnvironmentAwareness' stamp: 'KLG 11/13/2020 20:32:36'!
forEnvironment: anEnvironment
	"Answer new symbol for anEnvironment."

	^ SymbolManager ensureSymbol: self forEnvironment: anEnvironment! !

!Symbol methodsFor: '*EnvironmentAwareness' stamp: 'KLG 10/10/2020 21:11:07'!
forEnvironmentIfAware: anEnvironment
	"Answer an environment version of mine.
	
	I am environment aware, so convert me if necessary."
	
	^ self forEnvironment: anEnvironment! !

!Symbol methodsFor: '*EnvironmentAwareness' stamp: 'KLG 11/13/2020 22:16:12'!
forEnvironmentIfBound: anEnvironment
	"Answer an environment version of mine, if I am already bound in anEnvironment.
	
	Otherwise answe me."
	
	SymbolManager forSymbol: self inEnvironment: anEnvironment do: [ :boundSymbol |
		^ boundSymbol ]! !

!Symbol methodsFor: '*EnvironmentAwareness' stamp: 'KLG 11/13/2020 20:22:13'!
forGlobalNamespace
	"Answer global version of mine."
	
	^ Symbol lookup: self :: ifNil: [
		Symbol intern: self asString ]! !
